@@ -17,6 +17,7 @@ from solfege import mpd
 from solfege.mpd import mpdutils
 from solfege import cfg
 
+
 class TestParser(unittest.TestCase):
     def test_bad_musictype(self):
         s = 'question {\n' \
@@ -42,6 +43,27 @@ class TestParser(unittest.TestCase):
         self.assertEquals(p.m_questions[0].iname, "m7")
         self.assertEquals(p.m_questions[0].iname.cval, "chord|m7")
 
+class LessonfileTestCase(unittest.TestCase):
+    def _m(self, idx=0):
+        return self.p.m_questions[idx]['music']
+
+class TestMpdTransposable(unittest.TestCase):
+    def setUp(self):
+        cfg.set_bool('config/override_default_instrument', False)
+        self.p = QuestionsLessonfile()
+        self.p.parse_string("""
+        header { random_transpose = "atonal", -5, 5 }
+        question { music = chord("c e g") }
+        """)
+        self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
+        self.p._idx = 0
+    def test_get_musicdata_transposed(self):
+        self.assertEquals("d fis a", self.p.m_questions[0]['music'].get_musicdata_transposed(self.p))
+        self.p.header.random_transpose = ("key", -5, 6)
+        # get_musicdata_transposed works for all transposition modes.
+        self.assertEquals("d fis a", self.p.m_questions[0]['music'].get_musicdata_transposed(self.p))
+        self.assertEquals("c e g", self.p.m_questions[0]['music'].m_musicdata)
+
 class TestChord(unittest.TestCase):
     def setUp(self):
         cfg.set_bool('config/override_default_instrument', False)
@@ -55,18 +77,22 @@ class TestChord(unittest.TestCase):
     def get_get_mpd_music_string(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
             r"\staff{ <c e g> }")
-        self.p.header.random_transpose = True
+        self.p.header.random_transpose = (True,)
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
             r"\staff\transpose d'{ <c e g> }")
     def test_get_music_as_notename_list(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_music_as_notename_list(self.p), ['c', 'e', 'g'])
-        self.p.header.random_transpose = True
+        self.p.header.random_transpose = (True,)
         self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
         self.assertEquals(self.p.m_questions[0]['music'].get_music_as_notename_list(self.p), ['d', 'fis', 'a'])
     def test_get_lilypond_code(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), "\\score{  \\transpose c' d'{ <c e g> } \\layout {   ragged-last = ##t   \\context { \\Staff \\remove \"Time_signature_engraver\" }  }}")
+        self.p.header.random_transpose = ('atonal', -5, 5)
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), "\\score{    { <d fis a> } \\layout {   ragged-last = ##t   \\context { \\Staff \\remove \"Time_signature_engraver\" }  }}")
     def test_get_lilypond_code_first_note(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\transpose c' d'{ c }")
+        self.p.header.random_transpose = ('atonal', -5, 5)
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"{ d }")
     def test_play(self):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
@@ -77,20 +103,25 @@ class TestChord(unittest.TestCase):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:1 v0:121 n0:48 p1:2 v1:122 n1:52 p2:3 v2:123 n2:55 d1/4 o48 o52 o55")
+            "t60/4 p0:1 p1:2 p2:3 v0:121 n0:48 v1:122 n1:52 v2:123 n2:55 d1/4 o48 o52 o55")
     def test_play_arpeggio(self):
         question = self.p.m_questions[0]
         question['music'].play_arpeggio(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t180/4 p0:0 v0:100 n48 d1/4 o48 n52 d1/4 o52 n55 d1/4 o55")
-        self.p.header.random_transpose = True
+        self.p.header.random_transpose = (True,)
         question['music'].play_arpeggio(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t180/4 p0:0 v0:100 n50 d1/4 o50 n54 d1/4 o54 n57 d1/4 o57")
         cfg.set_bool('config/override_default_instrument', True)
         question['music'].play_arpeggio(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
-            "t180/4 p0:1 v0:121 n50 d1/4 o50 p0:2 v0:122 n54 d1/4 o54 p0:3 v0:123 n57 d1/4 o57")
+            "t180/4 p0:1 p1:2 p2:3 v0:121 n50 d1/4 o50 v1:122 n54 d1/4 o54 v2:123 n57 d1/4 o57")
+    def test_atonal_transpose(self):
+        self.p.header.random_transpose = ('atonal', -4, 4)
+        self.assertEquals(self.p.m_questions[0]['music'].get_musicdata_transposed(self.p), "d fis a")
+        self.p.m_transpose = mpd.MusicalPitch.new_from_notename("es'")
+        self.assertEquals(self.p.m_questions[0]['music'].get_musicdata_transposed(self.p), "ees g bes")
 
 class TestMusic(unittest.TestCase):
     def setUp(self):
@@ -121,7 +152,7 @@ class TestMusic(unittest.TestCase):
         question = self.p.m_questions[2]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:33 v1:34 n1:71 d1/4 o72 o71")
+            "t60/4 p0:37 p1:33 v0:38 n0:72 v1:34 n1:71 d1/4 o72 o71")
         # If the question has only one track, the last instrument,
         # representing the highest instrument, is used.
         question = self.p.m_questions[3]
@@ -145,7 +176,7 @@ class TestMusic(unittest.TestCase):
         # play with more than one instrument.:w
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:35 v1:36 n1:64 p2:33 v2:34 n2:48 d1/4 o72 o64 o48")
+            "t60/4 p0:37 p1:35 p2:33 v0:38 n0:72 v1:36 n1:64 v2:34 n2:48 d1/4 o72 o64 o48")
     def test_play_slowly(self):
         question = self.p.m_questions[0]
         #test.py TestMusic viser at det ikke blir generert riktig instrument
@@ -181,33 +212,67 @@ class TestMusic3(unittest.TestCase):
         cfg.set_bool('config/override_default_instrument', True)
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:3 v0:123 n0:72 p1:2 v1:122 n1:64 p2:1 v2:121 n2:48 d1/4 o72 o64 o48")
+            "t60/4 p0:3 p1:2 p2:1 v0:123 n0:72 v1:122 n1:64 v2:121 n2:48 d1/4 o72 o64 o48")
         #
         question = self.p.m_questions[1]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:35 v1:36 n1:67 n1:64 p2:33 v2:34 n2:48 d1/4 o72 o67 o64 o48")
+            "t60/4 p0:37 p1:35 p2:33 v0:38 n0:72 v1:36 n1:67 n1:64 v2:34 n2:48 d1/4 o72 o67 o64 o48")
 
-class TestVoice(unittest.TestCase):
+class TestVoiceCommon(unittest.TestCase):
+    def test_get_first_pitch(self):
+        for s, n in (
+            ("c d e", "c"),
+            ("<c'' e>", "c''"),
+            ("< c'' e>", "c''"),
+            (" c,,16", "c,,"),
+            (" \n c d e", "c"),
+            (r"\clef bass c d e", "c"),
+            (r"\clef violin \time 3/8 fis", "fis"),
+            (r'\clef "violin_8" d', "d"),
+            ("\\key g \\major \\time 2/4\n d'8 | [g g]", "d'"),
+            ("\nc", "c"),
+            ):
+            x = Rvoice(s)
+            self.assertEquals(x.get_first_pitch().get_octave_notename(), n)
+        self.assertRaises(MusicObjectException, lambda: Rvoice("\\clef bass").get_first_pitch())
+
+
+class TestVoice(LessonfileTestCase):
     def setUp(self):
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
         self.p.parse_string("""
         header { random_transpose = no }
         question { music = voice("c d e") }
+        question { music = voice("< e") }
         """)
         self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
         self.p._idx = 0
     def test_get_mpd_music_string(self):
-        self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            "\\staff{\nc d e\n}")
-        self.p.header.random_transpose = True
-        self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            "\\staff\\transpose d'{\nc d e\n}")
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
+                          "\\staff{\nc d e\n}")
+        self.p.header.random_transpose = [True]
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
+                          "\\staff\\transpose d'{\nc d e\n}")
+        self.p.header.random_transpose = ['atonal', -5, 6]
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
+                          "\\staff{\nd e fis\n}")
     def test_get_lilypond_code(self):
-        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), r"\transpose c' d'{ c d e }")
+        self.assertEquals(self._m().get_lilypond_code(self.p),
+                          r"{ c d e }")
+        self.p.header.random_transpose = [True]
+        self.assertEquals(self._m().get_lilypond_code(self.p),
+                          r"\transpose c' d'{ c d e }")
+        self.p.header.random_transpose = "atonal", -5, 6
+        self.assertEquals(self._m().get_lilypond_code(self.p),
+                          r"{ d e fis }")
     def test_get_lilypond_code_first_note(self):
-        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<<   \new Voice\transpose c' d'{ \cadenzaOn c }  \new Voice{ \hideNotes c d e }  >> \layout { ragged-last = ##t } }")
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<< \new Voice{ \cadenzaOn c } \new Voice{ \hideNotes c d e } >> \layout { ragged-last = ##t } }")
+        self.p.header.random_transpose = [True]
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), ur"\score{ \new Staff<< \new Voice\transpose c' d'{ \cadenzaOn c } \new Voice{ \hideNotes c d e } >> \layout { ragged-last = ##t } }")
+        self.p.header.random_transpose = "atonal", -5, 6
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<< \new Voice{ \cadenzaOn d } \new Voice{ \hideNotes c d e } >> \layout { ragged-last = ##t } }")
     def _test_play(self):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
@@ -244,6 +309,43 @@ class TestVoice(unittest.TestCase):
             x = Rvoice(s)
             self.assertEquals((p1, p2), mpdutils.find_possible_first_note(x.m_musicdata), "String failed: %s: %s" % (s, mpdutils.find_possible_first_note(x.m_musicdata)))
             self.assertEquals(s[p1:p2], n)
+    def test_error_handling(self):
+        m = self._m(1).get_mpd_music_string(self.p)
+        try:
+            mpd.parser.parse_to_score_object(m)
+        except mpd.MpdException, e:
+            self._m(1).complete_to_musicdata_coords(self.p, e)
+            ec = self._m(1).get_err_context(e, self.p)
+            self.assertEquals(ec, 
+              "Bad input to the music object of type voice made\n"
+              "the parser fail to parse the following generated\n"
+              "music code:\n"
+              "\\staff{\n"
+              "< e\n"
+              "}\n"
+              "^")
+
+        else:
+            self.fail("Test failed. No exception raised.")
+        return
+        try:
+            m = self._m(5).get_mpd_music_string(self.p)
+        except mpd.MpdException, e:
+            # We can not call complete_to_musicdata_coords becuase
+            # get_mpd_music_string promises to always raise exceptions
+            # relative to musicdata.
+            self.assertEquals(self._m(5).get_err_context(e, self.p),
+                "ERR f\n^^^")
+        else:
+            self.fail("Test failed. No exception raised.")
+        self.p.header.random_transpose = ('atonal', -4, 4)
+        try:
+            m = self._m(5).get_mpd_music_string(self.p)
+        except mpd.MpdException, e:
+            self.assertEquals(self._m(5).get_err_context(e, self.p),
+                "ERR f\n^^^")
+        else:
+            self.fail("Test failed. No exception raised.")
 
 class TestErrorHandling(unittest.TestCase):
     def setUp(self):
@@ -280,7 +382,7 @@ class TestErrorHandling(unittest.TestCase):
         cfg.set_bool('config/override_default_instrument', True)
         self._test_1()
     def _test_1(self):
-        self._dotest(("chord", "voice", "rvoice", "rhythm", "percussion", "rhythm"),
+        self._dotest(("chord", "voice",),
                    (("c d e ERR f",
                      "c d e ERR f\n"
                      "      ^^^"),
@@ -293,9 +395,6 @@ class TestErrorHandling(unittest.TestCase):
                     ("ERR",
                      "ERR\n"
                      "^^^"),
-                    ("4ERR",
-                     "4ERR\n"
-                     "^"),
                   ))
     def test_satb(self):
         self._test_satb()
@@ -346,23 +445,37 @@ class TestErrorHandling(unittest.TestCase):
             ))
 
 
-class TestRvoice(unittest.TestCase):
+class TestRvoice(LessonfileTestCase):
     def setUp(self):
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
         self.p.parse_string("""
         header { random_transpose = no }
         question { music = rvoice("c' d b c") }
-        question { music = rvoice("[c8' d] e16") }
+        question { music = rvoice("[c'8 d] e16") }
+        question { music = rvoice("c,, d' b c") }
+        question { music = rvoice("b,, d' b c") }
+        question { music = rvoice("< e") }
+        question { music = rvoice("ERR") }
+        question { music = rvoice("c d e f\ng ERR a") }
         """)
         self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
         self.p._idx = 0
     def test_get_mpd_music_string(self):
-        self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
             "\\staff\\relative c'{\nc d b c\n}")
-        self.p.header.random_transpose = True
-        self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            "\\staff\\transpose d'\\relative c'{\nc d b c\n}")
+        self.p.header.random_transpose = (True,)
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
+            "\\staff\\transpose d'\\relative c'{ c d b c }")
+        self.p.header.random_transpose = ['atonal', -5, 6]
+        self.assertEquals(self._m().get_mpd_music_string(self.p),
+            "\\staff\\relative d'{\nd e cis d\n}")
+        self.assertEquals(self._m(1).get_mpd_music_string(self.p),
+            "\\staff\\relative d'{\n[ d8 e ] fis16\n}")
+        self.assertEquals(self._m(2).get_mpd_music_string(self.p),
+            "\\staff\\relative d,,{\nd e' cis d\n}")
+        self.assertEquals(self._m(3).get_mpd_music_string(self.p),
+            "\\staff\\relative cis,{\ncis e' cis d\n}")
     def _test_play(self):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
@@ -385,9 +498,44 @@ class TestRvoice(unittest.TestCase):
     def test_get_lilypond_code(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), r"\transpose c' d'\relative c{ c' d b c }")
     def test_get_lilypond_code_first_note(self):
-        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<<   \new Voice\transpose c' d'\relative c{ \cadenzaOn c' }  \new Voice{ \hideNotes c' d b c }  >>  \layout { ragged-last = ##t } }")
-    def test_with_bream_first_note(self):
-        self.assertEquals(self.p.m_questions[1].music.get_mpd_music_string(self.p), "\\staff\\relative c{\n[c8' d] e16\n}")
+        self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<< \new Voice\transpose c' d'\relative c{ \cadenzaOn c' } \new Voice{ \hideNotes c' d b c } >> \layout { ragged-last = ##t } }")
+    def test_bad_first_note(self):
+        self.p.header.random_transpose = (True, 0, 0)
+        m = self._m(4).get_mpd_music_string(self.p)
+        try:
+            mpd.parser.parse_to_score_object(m)
+        except mpd.parser.ParseError, e:
+            self.assertEquals(self._m(4).get_err_context(e, self.p),
+                "Bad input to the music object of type rvoice made\n"
+                "the parser fail to parse the following generated\n"
+                "music code:\n"
+                "\\staff\\transpose d'\\relative e{ < e }\n"
+                "                                    ^")
+        else:
+            self.fail("Test failed. No exception raised.")
+        m = self._m(5).get_mpd_music_string(self.p)
+        try:
+            mpd.parser.parse_to_score_object(m)
+        except mpd.InvalidNotenameException, e:
+            self.assertEquals(self._m(5).get_err_context(e, self.p),
+                "Bad input to the music object of type rvoice made\n"
+                "the parser fail to parse the following generated\n"
+                "music code:\n"
+                "\\staff\\transpose d'\\relative ERR{ ERR }\n"
+                "                   ^^^^^^^^^^^^^")
+        else:
+            self.fail("Test failed. No exception raised.")
+        m = self._m(6).get_mpd_music_string(self.p)
+        try:
+            mpd.parser.parse_to_score_object(m)
+        except mpd.InvalidNotenameException, e:
+            self.assertEquals(self._m(6).get_err_context(e, self.p),
+                "Bad input to the music object of type rvoice made\n"
+                "the parser fail to parse the following generated\n"
+                "music code:\n"
+                "\\staff\\transpose d'\\relative c{ c d e f\n"
+                "g ERR a }\n"
+                "  ^^^")
 
 class TestSatb(unittest.TestCase):
     def setUp(self):
@@ -430,7 +578,7 @@ class TestSatb(unittest.TestCase):
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t60/4 p0:0 v0:100 n60 d1/4 o60 n55 d1/4 o55 n52 d1/4 o52 n55 d1/4 o55 n48 d1/4 o48")
         # Then with transposition
-        self.p.header.random_transpose = True
+        self.p.header.random_transpose = (True,)
         question['music'].play_arpeggio(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t60/4 p0:0 v0:100 n62 d1/4 o62 n57 d1/4 o57 n54 d1/4 o54 n57 d1/4 o57 n50 d1/4 o50")
@@ -439,8 +587,9 @@ class TestRhythm(unittest.TestCase):
     def setUp(self):
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.p.parse_string(r"""
         question { music = rhythm("d c8") }
+        question { music = rhythm("c \time 3\4 c d") }
         """)
         self.p._idx = 0
     def test_play(self):
@@ -448,6 +597,10 @@ class TestRhythm(unittest.TestCase):
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t60/4 v9:100 P80 d1/4 o80 P37 d1/8 o37")
+    def test_bad_time(self):
+        question = self.p.m_questions[1]
+        self.assertRaises(mpd.InvalidNotenameException,
+            question.music.get_mpd_music_string, self.p)
 
 class TestPercussion(unittest.TestCase):
     def setUp(self):
@@ -764,9 +917,11 @@ class TestLabelObject(unittest.TestCase):
 
 
 suite = unittest.makeSuite(TestParser)
+suite.addTest(unittest.makeSuite(TestMpdTransposable))
 suite.addTest(unittest.makeSuite(TestChord))
 suite.addTest(unittest.makeSuite(TestMusic))
 suite.addTest(unittest.makeSuite(TestMusic3))
+suite.addTest(unittest.makeSuite(TestVoiceCommon))
 suite.addTest(unittest.makeSuite(TestVoice))
 suite.addTest(unittest.makeSuite(TestRvoice))
 suite.addTest(unittest.makeSuite(TestSatb))
